/*
 * Copyright 2012 Hanson Robokind LLC.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.robokind.integration.motion_speech;

import java.io.File;
import java.util.Properties;
import java.util.logging.Level;
import java.util.logging.Logger;
import org.osgi.framework.BundleActivator;
import org.osgi.framework.BundleContext;
import org.robokind.api.common.lifecycle.ManagedService;
import org.robokind.api.common.lifecycle.ServiceLifecycleProvider;
import org.robokind.api.common.lifecycle.utils.SimpleLifecycle;
import org.robokind.api.common.osgi.lifecycle.ConfiguredServiceLifecycle;
import org.robokind.api.common.osgi.lifecycle.ConfiguredServiceParams;
import org.robokind.api.common.osgi.lifecycle.OSGiComponent;
import org.robokind.api.common.services.Constants;
import org.robokind.api.common.services.ServiceUtils;
import org.robokind.api.motion.Robot;
import org.robokind.api.speech.SpeechService;
import org.robokind.api.speech.viseme.VisemeBindingManager;
import org.robokind.api.speech.viseme.config.VisemeBindingManagerConfig;
import org.robokind.impl.speech.viseme.VisemeBindingManagerAvroConfigLoader;
import org.robokind.impl.speech.viseme.VisemeBindingManagerAvroConfigWriter;

/**
 *
 * @author Matthew Stevenson <www.robokind.org>
 */
public class Activator implements BundleActivator {
    private final static Logger theLogger = 
            Logger.getLogger(Activator.class.getName());
    
    @Override
    public void start(BundleContext context) throws Exception {
        theLogger.log(Level.INFO, "Speech-Motion integration Activation Begin.");
        ServiceUtils.registerConfigLoader(
                context, new VisemeBindingManagerAvroConfigLoader());
        ServiceUtils.registerConfigWriter(
                context, new VisemeBindingManagerAvroConfigWriter());
		// Does "myRobot" indicate "use default"?	If so, can we pull that as a constant from somewhere?
        // String robotName = "myRobot";
        // 2012-03-30 :  Stu temporarily set viseme-target robot name to match the Avatar.
        launchVisemes(context, "Avatar_ZenoR50");
        //"myRobot" is hardcoded for the physical robot
        //2012-04-09 : Matt is registering visemes for both
        launchVisemes(context, "myRobot");
    }
    
    private void launchVisemes(BundleContext context, String robotName){
        String visemeConfPath = getPath() + "resources/VisemeConf.json";
        theLogger.log(Level.INFO, "Opening Viseme Config: {0}", visemeConfPath);
        File confFile = new File(getPath() + "resources/VisemeConf.json");
        Robot.Id robotId = new Robot.Id(robotName);
        String speechServiceId = "speechService";
        String configFileId = "viseme/" + robotId + "/config/avro";
        launchConfiguredService(
                context, robotId, speechServiceId, configFileId);
        launchConfigFile(context, configFileId, confFile);
        launchVisemeFrameSource(context, robotId, speechServiceId);
        theLogger.log(Level.INFO, "Speech-Motion integration Activation Complete.");
    }
    
    private String getPath(){
        String brandingToken = "robokindcontroller";
        String cluster = "robokind";
        File f = new File("./");
        String path = f.getAbsolutePath();
        int len = path.length();
        if(len >= 5){
            String dir = path.substring(len-5, len-2).toLowerCase();
            if(dir.equals("bin")){
                return "../" + cluster + "/";
            }
        }
        int blen = brandingToken.length() + 2;
        if(len >= blen){
            String dir = path.substring(len-blen, len-2).toLowerCase();
            if(dir.equals(brandingToken)){
                return "./" + cluster + "/";
            }
        }
        return "./target/" + brandingToken + "/" + cluster + "/";
    }
    
    private ManagedService launchConfiguredService(
            BundleContext context, Robot.Id robotId, 
            String speechServiceId, String configFileId){
        ConfiguredServiceParams<
                VisemeBindingManager, 
                VisemeBindingManagerConfig, 
                File> params = new ConfiguredServiceParams(
                        VisemeBindingManager.class, 
                        VisemeBindingManagerConfig.class, 
                        File.class, null, null, configFileId, 
                        VisemeBindingManager.VERSION, 
                        VisemeBindingManagerAvroConfigLoader.VERSION);
        ConfiguredServiceLifecycle lifecycle = 
                new ConfiguredServiceLifecycle(params);
        Properties registrationProps = new Properties();
        registrationProps.put(Robot.PROP_ID, robotId.getRobtIdString());
        registrationProps.put(SpeechService.PROP_ID, speechServiceId);
        OSGiComponent comp = 
                new OSGiComponent(context, lifecycle, registrationProps);
        comp.start();
        return comp;
    }
    
    private ManagedService launchConfigFile(
            BundleContext context, String paramId, File file){
        Properties props = new Properties();
        props.put(Constants.CONFIG_PARAM_ID, paramId);
        props.put(Constants.CONFIG_FORMAT_VERSION, 
                VisemeBindingManagerAvroConfigLoader.VERSION.toString());
        ServiceLifecycleProvider lifecycle = 
                new SimpleLifecycle(file, File.class, props);
        OSGiComponent paramComp = new OSGiComponent(context, lifecycle);
        paramComp.start();
        return paramComp;
    }
    
    private ManagedService launchVisemeFrameSource(
            BundleContext context, Robot.Id robotId, String speechServiceId){
        OSGiComponent comp = new OSGiComponent(context,
                new VisemeFrameSourceLifecycle(robotId, speechServiceId));
        comp.start();
        return comp;
    }
    
    @Override
    public void stop(BundleContext context) throws Exception {
        //TODO add deactivation code here
    }
}
